#region License
// Copyright (c) 2005 Alexander Kolliopoulos
//
// This software is provided 'as-is', without any express or implied
// warranty. In no event will the authors be held liable for any damages
// arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose,
// including commercial applications, and to alter it and redistribute it
// freely, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must not
//    claim that you wrote the original software. If you use this software
//    in a product, an acknowledgment in the product documentation would
//    be appreciated but is not required.
//
// 2. Altered source versions must be plainly marked as such, and must not
//    be misrepresented as being the original software.
//
// 3. This notice may not be removed or altered from any source distribution.
#endregion

using System;
using System.Runtime.InteropServices;

namespace Lydos.Tablets
{
    /// <summary>
    /// Bindings for the unmanaged wintab32 DLL.
    /// </summary>
    internal static class NativeMethods
    {
        #region Structs
        [StructLayout(LayoutKind.Explicit, Pack = 4)]
        public struct DWord
        {
            [FieldOffset(0)]
            public UInt16 Lo;
            [FieldOffset(16)]
            public UInt16 Hi;
            [FieldOffset(0)]
            public UInt32 DW;
        }

        [StructLayout(LayoutKind.Sequential, Pack = 4)]
        public struct LoHi32
        {
            public UInt16 Lo;
            public UInt16 Hi;
        }

        [StructLayout(LayoutKind.Explicit, Pack = 4)]
        public struct Buttons
        {
            [FieldOffset(0)]
            public LoHi32 LH;
            [FieldOffset(0)]
            public Int32 DW;
        }

        [StructLayout(LayoutKind.Sequential, Pack = 4)]
        public struct Orientation
        {
            public Int16 Azimuth;
            public Int16 Altitude;
            public Int16 Twist;
        }

        [StructLayout(LayoutKind.Sequential, Pack = 4)]
        public struct Packet
        {
            public UInt32 Time;
            public Buttons Buttons;
            public Int32 X;
            public Int32 Y;
            public Int32 NormalPressure;
        }

        [StructLayout(LayoutKind.Sequential, Pack = 4)]
        public struct LogContext
        {
            [MarshalAs(UnmanagedType.ByValArray, SizeConst = 40)]
            public byte[] Name;
            public UInt32 Options;
            public UInt32 Status;
            public UInt32 Locks;
            public UInt32 MsgBase;
            public UInt32 Device;
            public UInt32 PktRate;
            public UInt32 PktData;
            public UInt32 PktMode;
            public UInt32 MoveMask;
            public UInt32 BtnDnMask;
            public UInt32 BtnUpMask;
            public Int32 InOrgX;
            public Int32 InOrgY;
            public Int32 InOrgZ;
            public Int32 InExtX;
            public Int32 InExtY;
            public Int32 InExtZ;
            public Int32 OutOrgX;
            public Int32 OutOrgY;
            public Int32 OutOrgZ;
            public Int32 OutExtX;
            public Int32 OutExtY;
            public Int32 OutExtZ;
            public UInt32 SensX;
            public UInt32 SensY;
            public UInt32 SensZ;
            public UInt32 SysMode;
            public Int32 SysOrgX;
            public Int32 SysOrgY;
            public Int32 SysExtX;
            public Int32 SysExtY;
            public UInt32 SysSensX;
            public UInt32 SysSensY;
        }
        #endregion

        #region Enums
        public enum WT
        {
            DefBase = 0x7FF0,
            MaxOffset = 0xF,
            Packet = DefBase + 0,
            CtxOpen = DefBase + 1,
            CtxClose = DefBase + 2,
            CtxUpdate = DefBase + 3,
            CtxOverlap = DefBase + 4,
            Proximity = DefBase + 5,
            InfoChange = DefBase + 6,
            CsrChange = DefBase + 7,
            Max = DefBase + MaxOffset
        }

        public enum PK
        {
            Context = 0x1,
            Status = 0x2,
            Time = 0x4,
            Changed = 0x8,
            SerialNumber = 0x10,
            Cursor = 0x20,
            Buttons = 0x40,
            X = 0x80,
            Y = 0x100,
            Z = 0x200,
            NormalPressure = 0x400,
            TangentPressure = 0x800,
            Orientation = 0x1000,
            Rotation = 0x2000,
            PacketData = (PK.X | PK.Y | PK.Buttons | PK.Time | PK.NormalPressure)
        }

        //public enum TU
        //{
        //    None = 0,
        //    Inches = 1,
        //    Centimeters = 2,
        //    Circle = 3
        //}

        //public enum SBN
        //{
        //    None = 0x0,
        //    LClick = 0x1,
        //    LDblClick = 0x2,
        //    LDrag = 0x3,
        //    RClick = 0x4,
        //    RDblClick = 0x5,
        //    RDrag = 0x6,
        //    MClick = 0x7,
        //    MDblClick = 0x8,
        //    MDrag = 0x9,
        //    PTClick = 0x10,
        //    PTDblClick = 0x20,
        //    PTDrag = 0x30,
        //    PNClick = 0x40,
        //    PNDblClick = 0x50,
        //    PNDrag = 0x60,
        //    P1Click = 0x70,
        //    P1DblClick = 0x80,
        //    P1Drag = 0x90,
        //    P2Click = 0xA0,
        //    P2DblClick = 0xB0,
        //    P2Drag = 0xC0,
        //    P3Click = 0xD0,
        //    P3DblClick = 0xE0,
        //    P3Drag = 0xF0
        //}

        //public enum HWC
        //{
        //    Integrated = 0x1,
        //    Touch = 0x2,
        //    HardProx = 0x4,
        //    PhysIDCursors = 0x8
        //}

        //public enum CRC
        //{
        //    MultiMode = 0x1,
        //    Aggregate = 0x2,
        //    Invert = 0x4
        //}

        public enum WTI
        {
            Interface = 1,
            Status = 2,
            DefContext = 3,
            DefSysCtx = 4,
            Devices = 100,
            Cursors = 200,
            Extensions = 300,
            DDCtxs = 400,
            DSCtxs = 500
        }

        //public enum IFC
        //{
        //    WintabID = 1,
        //    SpecVersion = 2,
        //    ImplVersion = 3,
        //    NDevices = 4,
        //    NCursors = 5,
        //    NContexts = 6,
        //    CtxOptions = 7,
        //    CtxSaveSize = 8,
        //    NExtensions = 9,
        //    NManagers = 10,
        //    Max = 10
        //}

        //public enum STA
        //{
        //    Contexts = 1,
        //    SysCtxs = 2,
        //    PktRate = 3,
        //    PktData = 4,
        //    Managers = 5,
        //    System = 6,
        //    ButtonUse = 7,
        //    SysBtnUse = 8,
        //    Max = 8
        //}

        //public enum CTX
        //{
        //    Name = 1,
        //    Options = 2,
        //    Status = 3,
        //    Locks = 4,
        //    MsgBase = 5,
        //    Device = 6,
        //    PktRate = 7,
        //    PktData = 8,
        //    PktMode = 9,
        //    MoveMask = 10,
        //    BtnDnMask = 11,
        //    BtnUpMask = 12,
        //    InOrgX = 13,
        //    InOrgY = 14,
        //    InOrgZ = 15,
        //    InExtX = 16,
        //    InExtY = 17,
        //    InExtZ = 18,
        //    OutOrgX = 19,
        //    OutOrgY = 20,
        //    OutOrgZ = 21,
        //    OutExtX = 22,
        //    OutExtY = 23,
        //    OutExtZ = 24,
        //    SensX = 25,
        //    SensY = 26,
        //    SensZ = 27,
        //    SysMode = 28,
        //    SysOrgX = 29,
        //    SysOrgY = 30,
        //    SysExtX = 31,
        //    SysExtY = 32,
        //    SysSensX = 33,
        //    SysSensY = 34,
        //    Max = 34
        //}

        public enum DVC
        {
            Name = 1,
            Hardware = 2,
            NCsrTypes = 3,
            FirstCsr = 4,
            PktRate = 5,
            PktData = 6,
            PktMode = 7,
            CSRData = 8,
            XMargin = 9,
            YMargin = 10,
            ZMargin = 11,
            X = 12,
            Y = 13,
            Z = 14,
            NPressure = 15,
            TPressure = 16,
            Orientation = 17,
            Rotation = 18,
            PnpID = 19,
            Max = 19
        }

        //public enum CSR
        //{
        //    Name = 1,
        //    Active = 2,
        //    PktData = 3,
        //    Buttons = 4,
        //    ButtonBits = 5,
        //    BtnNames = 6,
        //    ButtonMap = 7,
        //    SysBtnMap = 8,
        //    NPButton = 9,
        //    NPBtnMarks = 10,
        //    NPResponse = 11,
        //    TPButton = 12,
        //    TPBtnMarks = 13,
        //    TPResponse = 14,
        //    PhysID = 15,
        //    Mode = 16,
        //    MInPktData = 17,
        //    MInButtons = 18,
        //    Capabilities = 19,
        //    Max = 19
        //}

        //public enum EXT
        //{
        //    Name = 1,
        //    Tag = 2,
        //    Mask = 3,
        //    Size = 4,
        //    Axes = 5,
        //    Default = 6,
        //    DefContext = 7,
        //    DefSysCtx = 8,
        //    Cursors = 9,
        //    Max = 109
        //}

        public enum CXO
        {
            System = 0x1,
            Pen = 0x2,
            Messages = 0x4,
            Margin = 0x8000,
            MgnInside = 0x4000
        }

        //public enum CXS
        //{
        //    Disabled = 0x1,
        //    Obscured = 0x2,
        //    OnTop = 0x4
        //}

        //public enum CXL
        //{
        //    InSize = 0x1,
        //    InAspect = 0x2,
        //    Sensitivity = 0x4,
        //    Margin = 0x8,
        //    SysOut = 0x10
        //}

        //public enum TPS
        //{
        //    Proximity = 0x1,
        //    QueueErr = 0x2,
        //    Margin = 0x4,
        //    Grab = 0x8,
        //    Invert = 0x10
        //}

        //public enum TBN
        //{
        //    None = 0,
        //    Up = 1,
        //    Down = 2
        //}

        //public enum WTDC
        //{
        //    None = 0,
        //    Cancel = 1,
        //    OK = 2,
        //    Restart = 3
        //}

        //public enum WTH
        //{
        //    Playback = 1,
        //    Record = 2,
        //    GetLplpFn = -3,
        //    LplpFnNext = -2,
        //    LpFnNext = -1,
        //    Action = 0,
        //    GetNext = 1,
        //    Skip = 2
        //}
        #endregion

        #region DLL imports
        [DllImport("wintab32.dll")]
        [return: MarshalAs(UnmanagedType.Bool)]
        public static extern bool WTClose(IntPtr hCtx);

        //[DllImport("wintab32.dll")]
        //[return: MarshalAs(UnmanagedType.Bool)]
        //public static extern bool WTConfig(IntPtr hCtx, IntPtr hwnd);

        //[DllImport("wintab32.dll")]
        //public static extern int WTDataGet(IntPtr hCtx, uint wBegin, uint wEnd, int cMaxPackets, IntPtr lpPkts, IntPtr lpNPkts);

        //[DllImport("wintab32.dll")]
        //public static extern int WTDataPeek(IntPtr hCtx, uint wBegin, uint wEnding, int cMaxPackets, IntPtr lpPkts, IntPtr lpNPkts);

        [DllImport("wintab32.dll")]
        [return: MarshalAs(UnmanagedType.Bool)]
        public static extern bool WTEnable(IntPtr hCtx, [MarshalAs(UnmanagedType.Bool)] bool fEnable);

        //[DllImport("wintab32.dll")]
        //[return: MarshalAs(UnmanagedType.Bool)]
        //public static extern bool WTExtGet(IntPtr hCtx, int wExt, IntPtr lpData);

        //[DllImport("wintab32.dll")]
        //[return: MarshalAs(UnmanagedType.Bool)]
        //public static extern bool WTExtSet(IntPtr hCtx, int wExt, IntPtr lpData);

        //[DllImport("wintab32.dll", CharSet = CharSet.Ansi)]
        //[return: MarshalAs(UnmanagedType.Bool)]
        //public static extern bool WTGet(IntPtr hCtx, IntPtr LPLOGCONTEXT);

        [DllImport("wintab32.dll", CharSet = CharSet.Ansi)]
        public static extern uint WTInfo(int wCategory, int nIndex, IntPtr lpOutput);

        //[DllImport("wintab32.dll")]
        //[return: MarshalAs(UnmanagedType.Bool)]
        //public static extern bool WTMgrClose(IntPtr hMgr);

        //[DllImport("wintab32.dll")]
        //public static extern int WTMgrConfigReplaceExA(IntPtr hMgr, [MarshalAs(UnmanagedType.Bool)] bool fInstall, IntPtr lpszModule, IntPtr lpszCfgProc);

        //[DllImport("wintab32.dll")]
        //public static extern int WTMgrConfigReplaceExW(IntPtr hMgr, [MarshalAs(UnmanagedType.Bool)] bool fInstall, IntPtr lpszModule, IntPtr lpszCfgProc);

        //[DllImport("wintab32.dll")]
        //[return: MarshalAs(UnmanagedType.Bool)]
        //public static extern bool WTMgrContextEnum(IntPtr hMgr, IntPtr lpEnumFunc, IntPtr lParam);

        //[DllImport("wintab32.dll")]
        //public static extern int WTMgrContextOwner(IntPtr hMgr, IntPtr hCtx);

        //[DllImport("wintab32.dll")]
        //[return: MarshalAs(UnmanagedType.Bool)]
        //public static extern bool WTMgrCsrButtonMap(IntPtr hMgr, int wCursor, IntPtr lpLogBtns, IntPtr lpSysBtns);

        //[DllImport("wintab32.dll")]
        //[return: MarshalAs(UnmanagedType.Bool)]
        //public static extern bool WTMgrCsrEnable(IntPtr hMgr, int wCursor, [MarshalAs(UnmanagedType.Bool)] bool fEnable);

        //[DllImport("wintab32.dll")]
        //[return: MarshalAs(UnmanagedType.Bool)]
        //public static extern bool WTMgrCsrExt(IntPtr hMgr, int wCursor, int wExt, IntPtr lpData);

        //[DllImport("wintab32.dll")]
        //[return: MarshalAs(UnmanagedType.Bool)]
        //public static extern bool WTMgrCsrPressureBtnMarksEx(IntPtr hMgr, int wCursor, IntPtr lpNMarks, IntPtr lpTMarks);

        //[DllImport("wintab32.dll")]
        //[return: MarshalAs(UnmanagedType.Bool)]
        //public static extern bool WTMgrCsrPressureResponse(IntPtr hMgr, int wCursor, IntPtr lpNResp, IntPtr lpTResp);

        //[DllImport("wintab32.dll")]
        //public static extern int WTMgrDefContext(IntPtr hMgr, [MarshalAs(UnmanagedType.Bool)] bool fSystem);

        //[DllImport("wintab32.dll")]
        //public static extern int WTMgrDefContextEx(IntPtr hMgr, int wDevice, int fSystem);

        //[DllImport("wintab32.dll")]
        //public static extern uint WTMgrDeviceConfig(IntPtr hMgr, uint wDevice, IntPtr hwnd);

        //[DllImport("wintab32.dll")]
        //[return: MarshalAs(UnmanagedType.Bool)]
        //public static extern bool WTMgrExt(IntPtr hMgr, int wExt, IntPtr lpData);

        //[DllImport("wintab32.dll")]
        //public static extern IntPtr WTMgrOpen(IntPtr hwnd, uint wMsgBase);

        //[DllImport("wintab32.dll")]
        //public static extern int WTMgrPacketHookExA(IntPtr hMgr, int nType, IntPtr lpszModule, IntPtr lpszHookProc);

        //[DllImport("wintab32.dll")]
        //public static extern int WTMgrPacketHookExW(IntPtr hMgr, int nType, IntPtr lpszModule, IntPtr lpszHookProc);

        //[DllImport("wintab32.dll")]
        //public static extern int WTMgrPacketHookNext(int hHook, int nCode, IntPtr wParam, IntPtr lParam);

        //[DllImport("wintab32.dll")]
        //[return: MarshalAs(UnmanagedType.Bool)]
        //public static extern bool WTMgrPacketUnHook(int hHook);

        [DllImport("wintab32.dll", CharSet = CharSet.Ansi)]
        public static extern IntPtr WTOpen(IntPtr hwnd, IntPtr LPLOGCONTEXT, [MarshalAs(UnmanagedType.Bool)] bool fEnable);

        [DllImport("wintab32.dll")]
        [return: MarshalAs(UnmanagedType.Bool)]
        public static extern bool WTOverlap(IntPtr hCtx, [MarshalAs(UnmanagedType.Bool)] bool fToTop);

        [DllImport("wintab32.dll", CharSet = CharSet.Ansi)]
        [return: MarshalAs(UnmanagedType.Bool)]
        public static extern bool WTPacket(IntPtr hCtx, IntPtr wSerial, IntPtr lpPkts);

        //[DllImport("wintab32.dll", CharSet = CharSet.Ansi)]
        //public static extern IntPtr WTPacketsGet(IntPtr hCtx, int cMaxPackets, IntPtr lpPkts);

        //[DllImport("wintab32.dll")]
        //public static extern IntPtr WTPacketsPeek(IntPtr hCtx, int cMaxPackets, IntPtr lpPkts);

        //[DllImport("wintab32.dll")]
        //[return: MarshalAs(UnmanagedType.Bool)]
        //public static extern bool WTQueuePacketsEx(IntPtr hCtx, int lpOld, int lpNew);

        //[DllImport("wintab32.dll")]
        //public static extern int WTQueueSizeGet(IntPtr hCtx);

        //[DllImport("wintab32.dll")]
        //[return: MarshalAs(UnmanagedType.Bool)]
        //public static extern bool WTQueueSizeSet(IntPtr hCtx, int nPkts);

        //[DllImport("wintab32.dll")]
        //public static extern int WTRestore(IntPtr hwnd, IntPtr lpSaveInfo, [MarshalAs(UnmanagedType.Bool)] bool fEnable);

        //[DllImport("wintab32.dll")]
        //[return: MarshalAs(UnmanagedType.Bool)]
        //public static extern bool WTSave(IntPtr hCtx, IntPtr lpSaveInfo);

        //[DllImport("wintab32.dll", CharSet = CharSet.Ansi)]
        //[return: MarshalAs(UnmanagedType.Bool)]
        //public static extern bool WTSet(IntPtr hCtx, IntPtr lpLogContext);
        #endregion
    }
}
